// ===============================================
// File:                Tools.cxx
// Description:         often used functions
// Current:             Sept 28th 1999
// Author(s):           Thomas Kern
// ===============================================

/*
 *        thk VRML-Viewer (based on OpenGL)
 *  it shows static *.wrl files based on the VRML97 standard
 *
 *          Copyright (C) 1997-1999  Thomas Kern
 *                  All Rights Reserved
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * My email adress is: 
 *   Thomas.Kern@3Dsource.de 
 * 
 * You should find these files at: my homepage
 *   http://www.3Dsource.de
 *
 */


//=====================================================================
//      INCLUDES
//=====================================================================
//
#ifndef TOOLS_H
#define TOOLS_H

#include <stdio.h>
#include <string.h>
#include <math.h>

// messages
#ifdef _USE_FL
#include <FL/fl_message.H>
#endif // USE_FL

// math includes
#ifdef _WIN32
  #include <windows.h>
  #ifdef _MSC_VER // MS VCC
    #pragma warning(disable:4244) // warning += etc.
    #pragma warning(disable:4305)
  #endif
#else
  #ifdef __GNUC__
    typedef bool	boolean;
  #else
    enum boolean {false, true};
  #endif
#endif

// OpenGL includes
extern "C" {
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glut.h>
}

// JPEG includes
#define HAVE_BOOLEAN
extern "C" {
#include "jpeglib.h"
}


//=====================================================================
//	GLOBAL MATH DEFINES
//=====================================================================
//
typedef double	real;

#define THK_DEBUG
#define THK_PRINT_MESSAGES
#define THK_GERMAN_MESSAGES

/* Constants rounded for 21 decimals. */
#ifndef M_E
  #define M_E             2.71828182845904523536
#endif
#ifndef M_LOG2E 
  #define M_LOG2E         1.44269504088896340736
#endif 
#ifndef M_LOG10E  
  #define M_LOG10E        0.434294481903251827651
#endif
#ifndef M_LN2
  #define M_LN2           0.693147180559945309417
#endif
#ifndef M_LN10
  #define M_LN10          2.30258509299404568402
#endif
#ifndef M_PI
  #define M_PI            3.14159265358979323846
#endif
#ifndef M_PI_2
  #define M_PI_2          1.57079632679489661923
#endif
#ifndef M_PI_4
  #define M_PI_4          0.785398163397448309616
#endif
#ifndef M_1_PI
  #define M_1_PI          0.318309886183790671538
#endif
#ifndef M_2_PI
  #define M_2_PI          0.636619772367581343076
#endif
#ifndef M_1_SQRTPI
  #define M_1_SQRTPI      0.564189583547756286948
#endif
#ifndef M_2_SQRTPI
  #define M_2_SQRTPI      1.12837916709551257390
#endif
#ifndef M_SQRT2
  #define M_SQRT2         1.41421356237309504880
#endif
#ifndef M_SQRT_2
  #define M_SQRT_2        0.707106781186547524401
#endif


#define	   WORD_LENGHT       50
#define    NAMELENGHT       128 // max. Laenge des Dateinames einschl. Verzeichnis
#define    URL_SIZE         512 
#define    LINEWIDE         512
#define    MAX_DIR_ENTRIES   40 // Dateien in einem Verzeichnis
#define    MAXMESSAGELENGHT  1024
extern char myMessageString[MAXMESSAGELENGHT];


//=====================================================================
//  allgemeine Typ-Definitionen
//=====================================================================
//
#ifndef CALLBACK                // Win32 Konventionen
 #define CALLBACK 
#endif

//enum MatrixTyp     { myCEN, myTRA, myROT, myROTD };
enum DrawOption    { myCull, mySmooth, myTexture, myList };
//enum DrawStile     { FillON, LineON, PointON };
enum TextureMIN    { NearNear, NearLin, LinNear, LinLin };

//=====================================================================
// Die folgenden Funktionen lassen sich nicht einer Klasse zuordnen.
// Diese Callback-Funktionen werden von der GLU-Tesselation-Routine
// aufgerufen und veranlassen das korrekte Zeichnen der urspruenglich
// beliebig geformten Polygone als (konvexe) Dreiecke.
// DIE FELDGROESSE WIRD NICHT UEBERPRUEFT.
//=====================================================================

//---------------------------------------------------------------------
// Die Normalenvektoren
void CALLBACK VertexCall(void *vertexData);

//---------------------------------------------------------------------
// konkave Polygone aufsplitten und zeichnen (KEINE MEMBERFUNKTION)
void CALLBACK VertexAndColorCall(void *vertexData);

//---------------------------------------------------------------------
// konkave Polygone aufsplitten und zeichnen (KEINE MEMBERFUNKTION)
void CALLBACK VertexAndTextureCall(void *vertexData);

//---------------------------------------------------------------------
// konkave Polygone aufsplitten und zeichnen (KEINE MEMBERFUNKTION)
void CALLBACK CompleteValueCall(void *vertexData);

//---------------------------------------------------------------------
// Eine neues Polygon bzw. ein Polygonzug beginnen
void CALLBACK BeginNewPolygon(GLenum type);

//---------------------------------------------------------------------
// Den aktuellen Polygonzug beenden, die Normalenvektoren
// berechnen und das Zeichnen veranlassen
void CALLBACK DrawNewPolygon();


//=====================================================================
// allgemeine Funktionen
//=====================================================================

//---------------------------------------------------------------------
// Berechnung des Normalenvektors 
void CalcNormal(GLfloat *StartPoint, // Ursprung
                GLfloat *Point1,     // X-Achse
                GLfloat *Point2,     // Y-Achse
                GLfloat *Normal);    // Z-Achse wird berechnet

//---------------------------------------------------------------------
// wandelt die Zeichenkette string1 in eine Floatzahl um
void string2float(const char string[], GLfloat &zahl);

//---------------------------------------------------------------------
// Vektor und Matrixoperationen (Matrizen: rows == colums)
void dotProduct(int rows, GLfloat *result, GLfloat *v1, GLfloat *v2);
void addVectors(int rows, GLfloat *result, GLfloat *v1, GLfloat *v2);
void normalizeVect(GLfloat v[3]);
void multMatrix(GLfloat result[16], GLfloat m1[16], GLfloat m2[16]);
void multVector(GLfloat result[ 3], GLfloat m[16],  GLfloat v[3]);


//=====================================================================
// Bilddateien laden und fuer Nutzung als Textur vorbereiten
//=====================================================================
//
// return RGB-Feld, wird mit new erzeugt, delete ist Sache des Aufrufers
// Breite, Hoehe und image werden angepasst
unsigned char* loadPPMfile( char *file, int &Breite, int &Hoehe);
unsigned char* loadJPEGfile(char *file, int &Breite, int &Hoehe);

//=====================================================================
// Datei in verschiedenen Verzeichnissen suchen und bei Erfolg oeffnen
//=====================================================================
//
void ScanDirectory(char *dir, char *pattern,
                   char  filelist[MAX_DIR_ENTRIES][NAMELENGHT],
                   int  &filecount);
boolean CheckFile(char *file, char *modus);


//=====================================================================
//	INLINE MATH FUNCTIONS
//=====================================================================
//
//	sign(x)
//
// liefert das Vorzeichen (-1|1)
inline int sign( const int x )
{
  return (x<0) ? -1 : +1;
}

inline double sign( const double x )
{
  return (x<0.0) ? -1.0 : +1.0;
}

//-----------------------------------------------------------------------------
//	rad(x)
//
// Umrechnung von Grad nach Radiant
static const double h_rad = M_PI/180.0;

inline double rad( const double x )
{
  return h_rad*x;
}

//-----------------------------------------------------------------------------
//	grad(x)
//
// Umrechnung von Radiant nach Grad
static const double h_grad = 180.0/M_PI;

inline double grad( const double x )
{
  return h_grad*x;
}

//=====================================================================
//	MESSAGE FUNCTIONS
//=====================================================================
//
void showMessage();
void showMessage(char *string);
void writeMessage();
void writeMessage(char *string);


//=====================================================================
#endif // TOOLS_H
