// ===============================================
// File:                Tools.cxx
// Description:         often used functions
// Current:             Sept 28th 1999
// Author(s):           Thomas Kern
// ===============================================

/*
 *        thk VRML-Viewer (based on OpenGL)
 *  it shows static *.wrl files based on the VRML97 standard
 *
 *          Copyright (C) 1997-1999  Thomas Kern
 *                  All Rights Reserved
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Library General Public
 *   License as published by the Free Software Foundation; either
 *   version 2 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Library General Public License for more details.
 *
 *   You should have received a copy of the GNU Library General Public
 *   License along with this library; if not, write to the Free
 *   Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 *
 * My email adress is: 
 *   Thomas.Kern@3Dsource.de 
 * 
 * You should find these files at: my homepage
 *   http://www.3Dsource.de
 *
 */


//=====================================================================
//      INCLUDES
//=====================================================================
//
#include <math.h>
#include <stdio.h>
#include <string.h>

#ifndef _WIN32
 #include <dirent.h>  // directory operations
 #include <values.h>
#endif

#include "Tools.H"


//=====================================================================
// Die folgenden Funktionen lassen sich nicht einer Klasse zuordnen.
// Diese Callback-Funktionen werden von der GLU-Tesselation-Routine
// aufgerufen und veranlassen das korrekte Zeichnen der urspruenglich
// beliebig geformten Polygone als (konvexe) Dreiecke.
// DIE FELDGROESSE WIRD NICHT UEBERPRUEFT.
//=====================================================================

static enum   { TypeUndefinded, Triangle, TriangleFan,
                      TriangleStrip } myPolygonType;
//static int    myVertexCount;

//---------------------------------------------------------------------
// Die Normalenvektoren
void CALLBACK VertexCall(void *vertexData)
{
  GLfloat *myPointer; // entaelt in dieser Reihenfolge:
                      // Vertex, Normals, Color, Textur
  myPointer = (GLfloat*)(vertexData);
  glNormal3fv(myPointer+3);
  glVertex3fv(myPointer);
}

//---------------------------------------------------------------------
// konkave Polygone aufsplitten und zeichnen (KEINE MEMBERFUNKTION)
// DIE FELDGROESSE WIRD NICHT UEBERPRUEFT.
// Die Normalenvektoren werden immer verwendet.
void CALLBACK VertexAndColorCall(void *vertexData)
{
  GLfloat *myPointer; // entaelt in dieser Reihenfolge:
                      // Vertex, Normals, Color, Textur
  myPointer = (GLfloat*)(vertexData);
  glNormal3fv(myPointer+3);
  glColor3fv(myPointer+6);
  glVertex3fv(myPointer);
}

//---------------------------------------------------------------------
// konkave Polygone aufsplitten und zeichnen (KEINE MEMBERFUNKTION)
// DIE FELDGROESSE WIRD NICHT UEBERPRUEFT.
// Die Normalenvektoren werden immer verwendet.
void CALLBACK VertexAndTextureCall(void *vertexData)
{
  GLfloat *myPointer; // entaelt in dieser Reihenfolge:
                      // Vertex, Normals, Color, Textur
  myPointer = (GLfloat*)(vertexData);
  glNormal3fv(myPointer+3);
  glTexCoord2fv(myPointer+9);
  glVertex3fv(myPointer);
}

//---------------------------------------------------------------------
// konkave Polygone aufsplitten und zeichnen (KEINE MEMBERFUNKTION)
// DIE FELDGROESSE WIRD NICHT UEBERPRUEFT.
// Die Normalenvektoren werden immer verwendet.
void CALLBACK CompleteValueCall(void *vertexData)
{
  GLfloat *myPointer; // entaelt in dieser Reihenfolge:
                      // Vertex, Normals, Color, Textur
  myPointer = (GLfloat*)(vertexData);
  glNormal3fv(myPointer+3);
  glColor3fv(myPointer+6);
  glTexCoord2fv(myPointer+9);
  glVertex3fv(myPointer);
}

//---------------------------------------------------------------------
// Eine neues Polygon bzw. ein Polygonzug beginnen
void CALLBACK BeginNewPolygon(GLenum type)
{
  switch(type) {
  case GL_TRIANGLES:
    myPolygonType = Triangle;
    break;
  case GL_TRIANGLE_FAN:
    myPolygonType  = TriangleFan;
    //myVertexCount = 0;
    break;
  case GL_TRIANGLE_STRIP:
    myPolygonType  = TriangleStrip;
    //myVertexCount = 0;
    break;
  default:   myPolygonType = TypeUndefinded;
  }
  glBegin(type);  // nur vorlaeufig
}

//---------------------------------------------------------------------
// Den aktuellen Polygonzug beenden, die Normalenvektoren
// berechnen und das Zeichnen veranlassen
void CALLBACK DrawNewPolygon()
{
  switch(myPolygonType) {
  case Triangle:
    break;
  case TriangleFan:
    break;
  case TriangleStrip:
    break;
  case TypeUndefinded:  // avoid Warning
    break;
  }

  glEnd();
}


//=====================================================================
//  die folgenden Funktionen sind nicht an ein Objekt gebunden
//=====================================================================
//
// Berechnung eines Normalenvektors
void CalcNormal(GLfloat *StartPoint, // Ursprung
                GLfloat *Point1,     // X-Achse
                GLfloat *Point2,     // Y-Achse
                GLfloat *Normal)     // Z-Achse wird berechnet
{
  int       i;
  GLdouble  v1[3], v2[3];               // Richtungsvektoren

  for(i = 0; i < 3; i++) {
    v1[i] = (Point1[i] - StartPoint[i]);
    v2[i] = (Point2[i] - StartPoint[i]);
  }

  Normal[0] = v1[1]*v2[2] - v1[2]*v2[1];
  Normal[1] = v1[2]*v2[0] - v1[0]*v2[2];
  Normal[2] = v1[0]*v2[1] - v1[1]*v2[0];

  normalizeVect(Normal);
}

//---------------------------------------------------------------------
// String in Float-Zahl umwandeln
void string2float(const char word[], GLfloat &zahl)
{
  int      i, temp, len1;
  boolean  FloatZahl = false;

  len1 = strlen(word);

  for(i = 0; i < len1; i++) {
    if( word[i] == '.' || word[i] == 'e' || word[i] == 'E' )
      FloatZahl = true;             // Format ist Float
  }
  if(FloatZahl)
    sscanf(word, "%f", &zahl);
  else {
    sscanf(word, "%d", &temp);
    zahl = (GLfloat)temp;
  }
}

//=====================================================================
// Vektor und Matrixoperationen (Matrizen: rows == colums)
//=====================================================================
//
// Punktprodukt
void dotProduct(int rows, GLfloat *result, GLfloat *v1, GLfloat *v2)
{

}

//---------------------------------------------------------------------
// Vektoraddition
void addVectors(int rows, GLfloat *result, GLfloat *v1, GLfloat *v2)
{

}

//---------------------------------------------------------------------
// Vektoraddition
void normalizeVect(GLfloat v[3])
{
  int i;
  GLfloat length, invLength;

  length = sqrt( v[0]*v[0] + v[1]*v[1] + v[2]*v[2] );
  if(length) {
    invLength = 1/length;
    for(i = 0; i < 3; i++)
      v[i] *= invLength;
  }
}

//---------------------------------------------------------------------
// Matrizenmultiplikation
void multMatrix(GLfloat result[16], GLfloat m1[16], GLfloat m2[16])
{
  // die aktuellen Transformationen berechnen
  //     |a0  a4  a8   a12|
  //     |a1  a5  a9   a13|
  // M = |a2  a6  a10  a14|
  //     |a3  a7  a11  a15|
  // result = m1 * m2;
  result[ 0] = m1[ 0]*m2[0] + m1[ 1]*m2[4] + m1[ 2]*m2[ 8] + m1[ 3]*m2[12];
  result[ 1] = m1[ 0]*m2[1] + m1[ 1]*m2[5] + m1[ 2]*m2[ 9] + m1[ 3]*m2[13];
  result[ 2] = m1[ 0]*m2[2] + m1[ 1]*m2[6] + m1[ 2]*m2[10] + m1[ 3]*m2[14];
  result[ 3] = m1[ 0]*m2[3] + m1[ 1]*m2[7] + m1[ 2]*m2[11] + m1[ 3]*m2[15];
  result[ 4] = m1[ 4]*m2[0] + m1[ 5]*m2[4] + m1[ 6]*m2[ 8] + m1[ 7]*m2[12];
  result[ 5] = m1[ 4]*m2[1] + m1[ 5]*m2[5] + m1[ 6]*m2[ 9] + m1[ 7]*m2[13];
  result[ 6] = m1[ 4]*m2[2] + m1[ 5]*m2[6] + m1[ 6]*m2[10] + m1[ 7]*m2[14];
  result[ 7] = m1[ 4]*m2[3] + m1[ 5]*m2[7] + m1[ 6]*m2[11] + m1[ 7]*m2[15];
  result[ 8] = m1[ 8]*m2[0] + m1[ 9]*m2[4] + m1[10]*m2[ 8] + m1[11]*m2[12];
  result[ 9] = m1[ 8]*m2[1] + m1[ 9]*m2[5] + m1[10]*m2[ 9] + m1[11]*m2[13];
  result[10] = m1[ 8]*m2[2] + m1[ 9]*m2[6] + m1[10]*m2[10] + m1[11]*m2[14];
  result[11] = m1[ 8]*m2[3] + m1[ 9]*m2[7] + m1[10]*m2[11] + m1[11]*m2[15];
  result[12] = m1[12]*m2[0] + m1[13]*m2[4] + m1[14]*m2[ 8] + m1[15]*m2[12];
  result[13] = m1[12]*m2[1] + m1[13]*m2[5] + m1[14]*m2[ 9] + m1[15]*m2[13];
  result[14] = m1[12]*m2[2] + m1[13]*m2[6] + m1[14]*m2[10] + m1[15]*m2[14];
  result[15] = m1[12]*m2[3] + m1[13]*m2[7] + m1[14]*m2[11] + m1[15]*m2[15];
}

//---------------------------------------------------------------------
// Matrix mal Vektor
void multVector(GLfloat result[3], GLfloat m[16] , GLfloat v[3])
{
  result[0] = m[0]*v[0] + m[1]*v[1] + m[ 2]*v[2]; 
  result[1] = m[4]*v[0] + m[5]*v[1] + m[ 6]*v[2]; 
  result[2] = m[8]*v[0] + m[9]*v[1] + m[10]*v[2];
}


//=====================================================================
// Bilddateien laden und fuer Nutzung als Textur vorbereiten
//=====================================================================
//
// ppm-files
unsigned char* loadPPMfile(char *file, int &Breite, int &Hoehe)
{
  // nun ein paar lokale Variablen vereinbaren
  unsigned char   *ImagePtr;
  boolean          kommentar = true;
  int              FileTyp = 0, wert, testB, testH, BreiteMal3, j;
  long int         i, Feld, hilf;
  char             dummy[LINEWIDE+1];
  const int        len3 = WORD_LENGHT;
  char             word[len3+1];     // \0-Zeichen muss auch noch rein
  FILE            *Input  = NULL;
  boolean          skip = false;

  // erst mal Dateityp pruefen
  Input = fopen(file, "r");
  if( Input == NULL  ) {
    sprintf(myMessageString,"\nPPM-Textur '%s' nicht gefunden.", file);
    writeMessage();
  }
  else {
    Breite = Hoehe = 0;
    fscanf(Input, "%s", word);         // File-Typ muss im ersten String stehen
    if( !strcmp(word, "P3") )
      FileTyp = 3;
    else if( !strcmp(word, "P6") )
      FileTyp = 6;

    if(FileTyp) {                      // ist ppm-Datei
      sprintf(myMessageString, "\nLade Textur '%s'...", file);
      writeMessage();
      while(kommentar) {               // Kommentarzeilen ueberspringen
        fgets(dummy, LINEWIDE, Input); // auf naechste Zeile vorruecken
        fscanf(Input, "%s", word);
        if( strcmp(word, "#") && !(word[0] == '#') )
          kommentar = false;
      }
      sscanf(word, "%d", &Breite);     // letztes Zeichen entspricht der Bildbreite
      BreiteMal3 = 3*Breite;           // wird mehrmals gebraucht, da RGB
      fscanf(Input, "%d", &Hoehe);     // dieses dann der Hoehe
      testB = testH = 0;
      wert = 1;
      for(i = 1; i < 11; i++) {        // Zweier-Potenz testen
        wert *= 2;
        if(Breite  == wert)
          testB++;
        if(Hoehe == wert)
          testH++;
      }
      fgets(dummy, 127, Input);         // auf naechste Zeile vorruecken
      fgets(dummy, 127, Input);         // naechste Zeile wird nicht beachtet
      Feld = BreiteMal3*Hoehe;          // Datenfeld der richtigen Groesse anlegen
      if( Feld > MAXLONG )
        Feld = MAXLONG;

      // Bild einlesen
      ImagePtr = NULL;
      ImagePtr = new unsigned char[Feld]; // Feld anlegen (RGB)
      if(ImagePtr != NULL) {
        i = 0;
        switch(FileTyp) {
        case 3:                          // ASCII - Werte
          while( !feof(Input) && (i < Feld) ) {
            fscanf(Input, "%d", &wert);
            ImagePtr[i] = (unsigned char)wert;
            i++;
          }
          break;
        case 6:                          // Binaer
          // binaeres Format von Paintshop Pro macht Aerger... (=> xv unter Linux)
          fread(ImagePtr, sizeof(char), Feld, Input);
          break;
        } // end switch
        
        if( !skip ) {
          // Statusmeldungen und Warnungen ausgeben.
          if( Feld > MAXLONG )
            writeMessage("\nTextur-Datei zu gross, wird deshalb nur teilweise genutzt !");
            if(!testB) {
              sprintf(myMessageString,
                      "\tHinweis: Breite=%d ist keine Zweier-Potenz", Breite);
              writeMessage();
            }
            if(!testH) {
              sprintf(myMessageString,"\tHinweis: Hoehe=%d ist keine Zweier-Potenz", Hoehe);
              writeMessage();
            }

            // Werte noch spaltenweise spiegeln
            unsigned char *Spalte = new unsigned char[Hoehe]; // Hilfs-Feld anlegen (RGB)
            for(j = 0; j < BreiteMal3; j++) {
              for(i = 0; i < Hoehe; i++) {
                hilf = i*BreiteMal3;
                Spalte[i] = ImagePtr[hilf+j];               // Spalte komplett sichern
              }
              for(i = 0; i < Hoehe; i++) {
                hilf = i*BreiteMal3;
                ImagePtr[hilf+j] = Spalte[Hoehe -1-i];      // einmal drehen
              }
            }
            delete[] Spalte;          
        }
      }
    }
    else {
      sprintf(myMessageString, "\n Textur '%s' nicht im ppm-Format.", file);
      writeMessage();
    }
  }
  fclose(Input);

  return ImagePtr;
}


//---------------------------------------------------------------------
// jpeg-files
unsigned char* loadJPEGfile(char *file, int &Breite, int &Hoehe)
{
  unsigned char   *ImagePtr = NULL;

#ifdef _WIN32 // JPEG not supported under Linux today
  
  // nun ein paar lokale Variablen vereinbaren
  struct jpeg_decompress_struct   cinfo;
  struct jpeg_error_mgr           jerr;
  
  FILE            *Input;         // source file
  int              i, wert, testB, testH, BreiteMal3;
  long int         Feld, test;

  // erst mal Dateityp pruefen
  Input = fopen(file, "rb");
  if( Input == NULL  ) {
    sprintf(myMessageString,"\nJPEG-Textur '%s' nicht gefunden.", file);
    writeMessage();
  }
  else {
    sprintf(myMessageString, "\nLade Textur '%s'...", file);
    writeMessage();
    // set up the normal JPEG error routines
    cinfo.err = jpeg_std_error(&jerr);
    // Now we can initialize the JPEG decompression object.
    jpeg_create_decompress(&cinfo);
    // Step 2: specify data source (eg, a file)
    jpeg_stdio_src(&cinfo, Input);
    // Step 3: read file parameters with jpeg_read_header()
    jpeg_read_header(&cinfo, TRUE);
    // Step 4: set parameters for decompression
    // change any of the defaults set by jpeg_read_header()
    // scale_num, scale_denom:
    // only supported scaling ratios are 1/1, 1/2, 1/4, and 1/8
    // Step 5: Start decompressor
    jpeg_start_decompress(&cinfo);
    // now we know the image data:
    //  output_width                      image width and height, as scaled
          //  output_height
          //  out_color_components          # of color components in out_color_space
          //  output_components         # of color components returned per pixel
          //  colormap                        the selected colormap, if any
          //  actual_number_of_colors           number of entries in colormap
    // Image data is returned in top-to-bottom scanline order.
    Breite = cinfo.output_width;
    Hoehe  = cinfo.output_height;

          // Statusmeldungen und Warnungen ausgeben.
    wert = 1;
    for(i = 1; i < 11; i++) {        // Zweier-Potenz testen
      wert *= 2;
      if(Breite  == wert)
        testB++;
      if(Hoehe == wert)
        testH++;
    }
    if(!testB) {
      sprintf(myMessageString, "\tHinweis: Breite=%d ist keine Zweier-Potenz", Breite);
      writeMessage();
    }
    if(!testH) {
      sprintf(myMessageString,"\tHinweis: Hoehe=%d ist keine Zweier-Potenz", Hoehe);
      writeMessage();
    }
    
    // weiter geht's
    if(cinfo.output_components == 3) { // RGB
      BreiteMal3 = 3*Breite;
      Feld = BreiteMal3*Hoehe;            // Datenfeld der richtigen Groesse anlegen
      if( Feld > MAXLONG ) {
        Feld = MAXLONG;
        writeMessage("\nTextur-Datei zu gross, wird deshalb nur teilweise genutzt !");
      }     
      ImagePtr = new unsigned char[Feld]; // Feld anlegen (RGB)
      // Step 6: while (scan lines remain to be read) jpeg_read_scanlines(...);
      test = Feld - BreiteMal3;     // pruefen, ob noch Platz da ist
      unsigned char *rp = ImagePtr + test;  // gleichzeitig spiegeln
      while( (cinfo.output_scanline < cinfo.output_height) && (test >= 0) ) {
        jpeg_read_scanlines(&cinfo, &rp, 1);
        rp   -= BreiteMal3;   // 
        test -= BreiteMal3;
      }
    }
    // Step 7: Finish decompression
    jpeg_finish_decompress(&cinfo);
    // Step 8: Release JPEG decompression object
    jpeg_destroy_decompress(&cinfo);
  
    // fertig
    fclose(Input);
  }

#else // not WIN32
  writeMessage("\njpeg currently not supported.");
#endif // WIN32

  return ImagePtr;
}


//=====================================================================
// Dateien in verschiedenen Verzeichnissen suchen
//=====================================================================
//
static const int maxDir = 10;
static char scannedDir[maxDir][NAMELENGHT]; // speichert die bisher genutzten Verzeichnisse
static int  dirCount = 0;

//---------------------------------------------------------------------
// ein Verzeichnis unter auslesen
void ScanDirectory(char *dir, char *pattern,
                   char  filelist[MAX_DIR_ENTRIES][NAMELENGHT],
                   int  &filecount)
{
  char  wildcard[NAMELENGHT];
  int   lenDir;
  
  // Verzeichnisnamen: Laenge und Vollstaendigkeit feststellen
  lenDir = strlen(dir);
  if(lenDir) {
    if( dir[lenDir-1] != '/' )
      dir[lenDir-1] = '/';
  }

  // den Suchstring festlegen, Initialisierung
  filecount = 0;
  sprintf(wildcard, "%s%s", dir, pattern); // dir enthaelt bereits "/"

  // -----------------------
  // nun wird's OS-abhaengig
#ifdef _WIN32

  HANDLE            searchPath = NULL;
  // LPCTSTR ist const char *
  WIN32_FIND_DATA   dataStruct;
  
  searchPath = FindFirstFile( (LPCTSTR)wildcard, &dataStruct);
  // Dateinamen festhalten
  if( searchPath != INVALID_HANDLE_VALUE ) {
    strcpy(filelist[filecount], dir);
    strcat(filelist[filecount], dataStruct.cFileName);
    filecount++;
    // auf zu den naechsten
    while(    filecount < MAX_DIR_ENTRIES // Grenzen beachten
           && FindNextFile(searchPath, &dataStruct) ) { // liefert BOOL
      strcpy(filelist[filecount], dir);
      strcat(filelist[filecount], dataStruct.cFileName);
      filecount++;
    }
    FindClose(searchPath);
  }

#else // Unix
  DIR           *searchPath = NULL;
  struct dirent *dataStruct;
  int            i, test, lenf, lenp;
  char          *file;

  // Verzeichnis oeffnen
  if( (searchPath = opendir(dir)) ) { 
    while(    filecount < MAX_DIR_ENTRIES // Grenzen beachten
           && (dataStruct = readdir(searchPath)) ) {
      file = dataStruct->d_name;
      lenf = strlen(file);      
      lenp = strlen(pattern);
      test = 0;
      for(i = 1; i < 4; i++) {
              if( file[lenf-i] == pattern[lenp-i] )
                test++;
      }
      if( test == 3 ) { // Dateiendung stimmt ueberein
              strcpy(filelist[filecount], dir);
              strcat(filelist[filecount], dataStruct->d_name);
              filecount++;
      }
    }
    closedir(searchPath);
  }
#endif 
}

//---------------------------------------------------------------------
// Verzeichnis und Dateiname trennen
static void extractDir(char *complete, char *directory, char *file)
{
  char    tempName[NAMELENGHT];
  int     len, i, j;
  boolean haveDir = false;

  strcpy(directory, complete);
  len = strlen(directory);

  // falls kein Vezeichnis angegeben, den Rest ueberspringen
  for(i = 0; i < len; i++) {
    if( directory[i] == '/' ) {
      haveDir = true;
      break;
    }
  }

  if(!haveDir) {
    directory[0] = '\0';
    strcpy(file, complete);
  }
  else {
    // Verzeichnis und Dateinamen trennen
    i = len-1; 
    j = 0;
    while( i && directory[i] != '/' ) {
      tempName[j]  = directory[i];  // vor dem Loeschen noch speichern
      j++;
      directory[i] = '\0';
      i--;
    }

    // Dateiname muss noch in die richtige Reihenfolge gebracht werden
    tempName[j] = '\0'; // String zuende
    len = strlen(tempName);
    for(i = 0; i < len; i++) 
      file[i] = tempName[len-i-1];
    file[i] = '\0';
  
    // Verzeichnis fuer spaeteres Durchsuchen speichern
    if(!dirCount) { // erstmaliger Aufruf
      strcpy(scannedDir[dirCount], directory);
      dirCount++;
    }
    else if(dirCount < maxDir) {
      i = 0;
      j = 0;  
      while(i < dirCount) {
        j += strcmp(scannedDir[i], directory); // Verzeichnis schon bekannt ?
        i++;
      }
      if(j) {                                  // Antwort ist: nein
        strcpy(scannedDir[dirCount], directory);
        dirCount++;
      }
    }
  }
}

//---------------------------------------------------------------------
// eine Datei oeffnen bzw. suchen
boolean CheckFile(char *file, char *modus)
{
  FILE    *Input = NULL;
  boolean  found = false;
  char     dirName[NAMELENGHT], fileName[NAMELENGHT];
  char     searchFile[NAMELENGHT];
  int      i;

  extractDir(file, dirName, fileName);
  i = 0;
  while( (Input == NULL) && (i < dirCount) ) {
    searchFile[0] = '\0'; // String loeschen
    strcat(searchFile, scannedDir[i]);
    strcat(searchFile, fileName);
    strcpy(file, searchFile);
    Input = fopen(file, modus);
    i++;
  }
  if( Input == NULL ) {              // nicht in Unterverzeichnissen
    strcpy(file, fileName);
    Input = fopen(file, modus);
  }
    
  // letztmalig ueberprufen 
  if(Input) {
    fclose(Input);
    found = true;
  }
  return found;
}

//=====================================================================
//	MESSAGE FUNCTIONS
//=====================================================================
//
char myMessageString[MAXMESSAGELENGHT];

//---------------------------------------------------------------------
//
void showMessage()
{
  #ifdef _USE_FL
  fl_message(myMessageString);
  #elif _CONSOLE
    printf("%s\n", myMessageString); fflush(stdout);
  #elif _USE_WIN32

  #endif
}

void showMessage(char *string)
{
  sprintf(myMessageString, string);
  showMessage();
}

//---------------------------------------------------------------------
//
void writeMessage()
{
  FILE *out = NULL;
  static boolean created = false;

  if(!created) {
    // Datei nach Programmstart neu anlegen
    out = fopen("log.thk", "wt");
    // testen, ob Datei angelegt werden konnte
    if( out == NULL ) {
      showMessage("Log-File could not created.");
    }
    else {
      created = true;
      fprintf(out, "%s", myMessageString);
      fclose(out);
    }
  }
  else {
    out = fopen("log.thk", "at");
    fprintf(out, "%s", myMessageString);
    fclose(out);
  }
}

void writeMessage(char *string)
{
  sprintf(myMessageString, string); 
  writeMessage();
}

//=====================================================================
